@@ -1,99 +1,152 @@
+import re
+
+from sqlalchemy import schema
+from sqlalchemy import types as sqltypes
 from sqlalchemy.ext.compiler import compiles
-from sqlalchemy import types as sqltypes
-from sqlalchemy import schema
 
-from ..util.compat import string_types
-from .. import util
-from .impl import DefaultImpl
-from .base import ColumnNullable, ColumnName, ColumnDefault, \
-    ColumnType, AlterColumn, format_column_name, \
-    format_server_default
 from .base import alter_table
+from .base import AlterColumn
+from .base import ColumnDefault
+from .base import ColumnName
+from .base import ColumnNullable
+from .base import ColumnType
+from .base import format_column_name
+from .base import format_server_default
+from .impl import DefaultImpl
+from .. import util
 from ..autogenerate import compare
+from ..util.compat import string_types
+from ..util.sqla_compat import _is_type_bound
+from ..util.sqla_compat import sqla_100
 
 
 class MySQLImpl(DefaultImpl):
-    __dialect__ = 'mysql'
+    __dialect__ = "mysql"
 
     transactional_ddl = False
 
-    def alter_column(self, table_name, column_name,
-                     nullable=None,
-                     server_default=False,
-                     name=None,
-                     type_=None,
-                     schema=None,
-                     existing_type=None,
-                     existing_server_default=None,
-                     existing_nullable=None,
-                     autoincrement=None,
-                     existing_autoincrement=None,
-                     **kw
-                     ):
+    def alter_column(
+        self,
+        table_name,
+        column_name,
+        nullable=None,
+        server_default=False,
+        name=None,
+        type_=None,
+        schema=None,
+        existing_type=None,
+        existing_server_default=None,
+        existing_nullable=None,
+        autoincrement=None,
+        existing_autoincrement=None,
+        comment=False,
+        existing_comment=None,
+        **kw
+    ):
         if name is not None:
             self._exec(
                 MySQLChangeColumn(
-                    table_name, column_name,
+                    table_name,
+                    column_name,
                     schema=schema,
                     newname=name,
-                    nullable=nullable if nullable is not None else
-                    existing_nullable
+                    nullable=nullable
+                    if nullable is not None
+                    else existing_nullable
                     if existing_nullable is not None
                     else True,
                     type_=type_ if type_ is not None else existing_type,
-                    default=server_default if server_default is not False
+                    default=server_default
+                    if server_default is not False
                     else existing_server_default,
-                    autoincrement=autoincrement if autoincrement is not None
-                    else existing_autoincrement
+                    autoincrement=autoincrement
+                    if autoincrement is not None
+                    else existing_autoincrement,
                 )
             )
-        elif nullable is not None or \
-                type_ is not None or \
-                autoincrement is not None:
+        elif (
+            nullable is not None
+            or type_ is not None
+            or autoincrement is not None
+            or comment is not False
+        ):
             self._exec(
                 MySQLModifyColumn(
-                    table_name, column_name,
+                    table_name,
+                    column_name,
                     schema=schema,
                     newname=name if name is not None else column_name,
-                    nullable=nullable if nullable is not None else
-                    existing_nullable
+                    nullable=nullable
+                    if nullable is not None
+                    else existing_nullable
                     if existing_nullable is not None
                     else True,
                     type_=type_ if type_ is not None else existing_type,
-                    default=server_default if server_default is not False
+                    default=server_default
+                    if server_default is not False
                     else existing_server_default,
-                    autoincrement=autoincrement if autoincrement is not None
-                    else existing_autoincrement
+                    autoincrement=autoincrement
+                    if autoincrement is not None
+                    else existing_autoincrement,
+                    comment=comment
+                    if comment is not False
+                    else existing_comment,
                 )
             )
         elif server_default is not False:
             self._exec(
                 MySQLAlterDefault(
-                    table_name, column_name, server_default,
-                    schema=schema,
+                    table_name, column_name, server_default, schema=schema
                 )
             )
 
-    def compare_server_default(self, inspector_column,
-                               metadata_column,
-                               rendered_metadata_default,
-                               rendered_inspector_default):
+    def drop_constraint(self, const):
+        if isinstance(const, schema.CheckConstraint) and _is_type_bound(const):
+            return
+
+        super(MySQLImpl, self).drop_constraint(const)
+
+    def compare_server_default(
+        self,
+        inspector_column,
+        metadata_column,
+        rendered_metadata_default,
+        rendered_inspector_default,
+    ):
         # partially a workaround for SQLAlchemy issue #3023; if the
         # column were created without "NOT NULL", MySQL may have added
         # an implicit default of '0' which we need to skip
-        if metadata_column.type._type_affinity is sqltypes.Integer and \
-            inspector_column.primary_key and \
-                not inspector_column.autoincrement and \
-                not rendered_metadata_default and \
-                rendered_inspector_default == "'0'":
+        # TODO: this is not really covered anymore ?
+        if (
+            metadata_column.type._type_affinity is sqltypes.Integer
+            and inspector_column.primary_key
+            and not inspector_column.autoincrement
+            and not rendered_metadata_default
+            and rendered_inspector_default == "'0'"
+        ):
             return False
+        elif inspector_column.type._type_affinity is sqltypes.Integer:
+            rendered_inspector_default = re.sub(
+                r"^'|'$", "", rendered_inspector_default
+            ) if rendered_inspector_default is not None else None
+            return rendered_inspector_default != rendered_metadata_default
+        elif rendered_inspector_default and rendered_metadata_default:
+            # adjust for "function()" vs. "FUNCTION"
+            return re.sub(
+                r"(.*?)(?:\(\))?$", r"\1", rendered_inspector_default.lower()
+            ) != re.sub(
+                r"(.*?)(?:\(\))?$", r"\1", rendered_metadata_default.lower()
+            )
         else:
             return rendered_inspector_default != rendered_metadata_default
 
-    def correct_for_autogen_constraints(self, conn_unique_constraints,
-                                        conn_indexes,
-                                        metadata_unique_constraints,
-                                        metadata_indexes):
+    def correct_for_autogen_constraints(
+        self,
+        conn_unique_constraints,
+        conn_indexes,
+        metadata_unique_constraints,
+        metadata_indexes,
+    ):
 
         # TODO: if SQLA 1.0, make use of "duplicates_index"
         # metadata
@@ -125,23 +178,46 @@ class MySQLImpl(DefaultImpl):
             if idx.name in removed:
                 metadata_indexes.remove(idx)
 
+        if not sqla_100:
+            self._legacy_correct_for_dupe_uq_uix(
+                conn_unique_constraints,
+                conn_indexes,
+                metadata_unique_constraints,
+                metadata_indexes,
+            )
+
+    def _legacy_correct_for_dupe_uq_uix(
+        self,
+        conn_unique_constraints,
+        conn_indexes,
+        metadata_unique_constraints,
+        metadata_indexes,
+    ):
+
         # then dedupe unique indexes vs. constraints, since MySQL
         # doesn't really have unique constraints as a separate construct.
         # but look in the metadata and try to maintain constructs
         # that already seem to be defined one way or the other
         # on that side.  See #276
-        metadata_uq_names = set([
-            cons.name for cons in metadata_unique_constraints
-            if cons.name is not None])
+        metadata_uq_names = set(
+            [
+                cons.name
+                for cons in metadata_unique_constraints
+                if cons.name is not None
+            ]
+        )
 
-        unnamed_metadata_uqs = set([
-            compare._uq_constraint_sig(cons).sig
-            for cons in metadata_unique_constraints
-            if cons.name is None
-        ])
+        unnamed_metadata_uqs = set(
+            [
+                compare._uq_constraint_sig(cons).sig
+                for cons in metadata_unique_constraints
+                if cons.name is None
+            ]
+        )
 
-        metadata_ix_names = set([
-            cons.name for cons in metadata_indexes if cons.unique])
+        metadata_ix_names = set(
+            [cons.name for cons in metadata_indexes if cons.unique]
+        )
         conn_uq_names = dict(
             (cons.name, cons) for cons in conn_unique_constraints
         )
@@ -151,16 +227,44 @@ class MySQLImpl(DefaultImpl):
 
         for overlap in set(conn_uq_names).intersection(conn_ix_names):
             if overlap not in metadata_uq_names:
-                if compare._uq_constraint_sig(conn_uq_names[overlap]).sig \
-                        not in unnamed_metadata_uqs:
+                if (
+                    compare._uq_constraint_sig(conn_uq_names[overlap]).sig
+                    not in unnamed_metadata_uqs
+                ):
 
                     conn_unique_constraints.discard(conn_uq_names[overlap])
             elif overlap not in metadata_ix_names:
                 conn_indexes.discard(conn_ix_names[overlap])
 
+    def correct_for_autogen_foreignkeys(self, conn_fks, metadata_fks):
+        conn_fk_by_sig = dict(
+            (compare._fk_constraint_sig(fk).sig, fk) for fk in conn_fks
+        )
+        metadata_fk_by_sig = dict(
+            (compare._fk_constraint_sig(fk).sig, fk) for fk in metadata_fks
+        )
+
+        for sig in set(conn_fk_by_sig).intersection(metadata_fk_by_sig):
+            mdfk = metadata_fk_by_sig[sig]
+            cnfk = conn_fk_by_sig[sig]
+            # MySQL considers RESTRICT to be the default and doesn't
+            # report on it.  if the model has explicit RESTRICT and
+            # the conn FK has None, set it to RESTRICT
+            if (
+                mdfk.ondelete is not None
+                and mdfk.ondelete.lower() == "restrict"
+                and cnfk.ondelete is None
+            ):
+                cnfk.ondelete = "RESTRICT"
+            if (
+                mdfk.onupdate is not None
+                and mdfk.onupdate.lower() == "restrict"
+                and cnfk.onupdate is None
+            ):
+                cnfk.onupdate = "RESTRICT"
+
 
 class MySQLAlterDefault(AlterColumn):
-
     def __init__(self, name, column_name, default, schema=None):
         super(AlterColumn, self).__init__(name, schema=schema)
         self.column_name = column_name
@@ -168,19 +272,25 @@ class MySQLAlterDefault(AlterColumn):
 
 
 class MySQLChangeColumn(AlterColumn):
-
-    def __init__(self, name, column_name, schema=None,
-                 newname=None,
-                 type_=None,
-                 nullable=None,
-                 default=False,
-                 autoincrement=None):
+    def __init__(
+        self,
+        name,
+        column_name,
+        schema=None,
+        newname=None,
+        type_=None,
+        nullable=None,
+        default=False,
+        autoincrement=None,
+        comment=False,
+    ):
         super(AlterColumn, self).__init__(name, schema=schema)
         self.column_name = column_name
         self.nullable = nullable
         self.newname = newname
         self.default = default
         self.autoincrement = autoincrement
+        self.comment = comment
         if type_ is None:
             raise util.CommandError(
                 "All MySQL CHANGE/MODIFY COLUMN operations "
@@ -194,10 +304,10 @@ class MySQLModifyColumn(MySQLChangeColumn):
     pass
 
 
-@compiles(ColumnNullable, 'mysql')
-@compiles(ColumnName, 'mysql')
-@compiles(ColumnDefault, 'mysql')
-@compiles(ColumnType, 'mysql')
+@compiles(ColumnNullable, "mysql")
+@compiles(ColumnName, "mysql")
+@compiles(ColumnDefault, "mysql")
+@compiles(ColumnType, "mysql")
 def _mysql_doesnt_support_individual(element, compiler, **kw):
     raise NotImplementedError(
         "Individual alter column constructs not supported by MySQL"
@@ -211,7 +321,7 @@ def _mysql_alter_default(element, compiler, **kw):
         format_column_name(compiler, element.column_name),
         "SET DEFAULT %s" % format_server_default(compiler, element.default)
         if element.default is not None
-        else "DROP DEFAULT"
+        else "DROP DEFAULT",
     )
 
 
@@ -225,7 +335,8 @@ def _mysql_modify_column(element, compiler, **kw):
             nullable=element.nullable,
             server_default=element.default,
             type_=element.type_,
-            autoincrement=element.autoincrement
+            autoincrement=element.autoincrement,
+            comment=element.comment,
         ),
     )
 
@@ -241,7 +352,8 @@ def _mysql_change_column(element, compiler, **kw):
             nullable=element.nullable,
             server_default=element.default,
             type_=element.type_,
-            autoincrement=element.autoincrement
+            autoincrement=element.autoincrement,
+            comment=element.comment,
         ),
     )
 
@@ -253,16 +365,21 @@ def _render_value(compiler, expr):
         return compiler.sql_compiler.process(expr)
 
 
-def _mysql_colspec(compiler, nullable, server_default, type_,
-                   autoincrement):
+def _mysql_colspec(
+    compiler, nullable, server_default, type_, autoincrement, comment
+):
     spec = "%s %s" % (
         compiler.dialect.type_compiler.process(type_),
-        "NULL" if nullable else "NOT NULL"
+        "NULL" if nullable else "NOT NULL",
     )
     if autoincrement:
         spec += " AUTO_INCREMENT"
     if server_default is not False and server_default is not None:
         spec += " DEFAULT %s" % _render_value(compiler, server_default)
+    if comment:
+        spec += " COMMENT %s" % compiler.sql_compiler.render_literal_value(
+            comment, sqltypes.String()
+        )
 
     return spec
 
@@ -273,17 +390,25 @@ def _mysql_drop_constraint(element, compiler, **kw):
     raise errors for invalid constraint type."""
 
     constraint = element.element
-    if isinstance(constraint, (schema.ForeignKeyConstraint,
-                               schema.PrimaryKeyConstraint,
-                               schema.UniqueConstraint)
-                  ):
+    if isinstance(
+        constraint,
+        (
+            schema.ForeignKeyConstraint,
+            schema.PrimaryKeyConstraint,
+            schema.UniqueConstraint,
+        ),
+    ):
         return compiler.visit_drop_constraint(element, **kw)
     elif isinstance(constraint, schema.CheckConstraint):
-        raise NotImplementedError(
-            "MySQL does not support CHECK constraints.")
+        # note that SQLAlchemy as of 1.2 does not yet support
+        # DROP CONSTRAINT for MySQL/MariaDB, so we implement fully
+        # here.
+        return "ALTER TABLE %s DROP CONSTRAINT %s" % (
+            compiler.preparer.format_table(constraint.table),
+            compiler.preparer.format_constraint(constraint),
+        )
     else:
         raise NotImplementedError(
             "No generic 'DROP CONSTRAINT' in MySQL - "
-            "please specify constraint type")
-
-
+            "please specify constraint type"
+        )
