@@ -1,10 +1,13 @@
-from alembic import util
+import sys
 
+from alembic import util
+from alembic.util import sqla_compat
 from . import exclusions
 
 if util.sqla_094:
     from sqlalchemy.testing.requirements import Requirements
 else:
+
     class Requirements(object):
         pass
 
@@ -20,12 +23,15 @@ class SuiteRequirements(Requirements):
     @property
     def unique_constraint_reflection(self):
         def doesnt_have_check_uq_constraints(config):
-            if not util.sqla_084:
-                return True
             from sqlalchemy import inspect
+
+            # temporary
+            if config.db.name == "oracle":
+                return True
+
             insp = inspect(config.db)
             try:
-                insp.get_unique_constraints('x')
+                insp.get_unique_constraints("x")
             except NotImplementedError:
                 return True
             except TypeError:
@@ -34,17 +40,11 @@ class SuiteRequirements(Requirements):
                 pass
             return False
 
-        return exclusions.skip_if(
-            lambda config: not util.sqla_084,
-            "SQLAlchemy 0.8.4 or greater required"
-        ) + exclusions.skip_if(doesnt_have_check_uq_constraints)
+        return exclusions.skip_if(doesnt_have_check_uq_constraints)
 
     @property
     def foreign_key_match(self):
-        return exclusions.fails_if(
-            lambda config: not util.sqla_08,
-            "MATCH for foreign keys added in SQLAlchemy 0.8.0"
-        )
+        return exclusions.open()
 
     @property
     def check_constraints_w_enforcement(self):
@@ -61,94 +61,119 @@ class SuiteRequirements(Requirements):
     def reflects_fk_options(self):
         return exclusions.closed()
 
-    @property
-    def fail_before_sqla_079(self):
-        return exclusions.fails_if(
-            lambda config: not util.sqla_079,
-            "SQLAlchemy 0.7.9 or greater required"
-        )
-
-    @property
-    def fail_before_sqla_080(self):
-        return exclusions.fails_if(
-            lambda config: not util.sqla_08,
-            "SQLAlchemy 0.8.0 or greater required"
-        )
-
-    @property
-    def fail_before_sqla_083(self):
-        return exclusions.fails_if(
-            lambda config: not util.sqla_083,
-            "SQLAlchemy 0.8.3 or greater required"
-        )
-
-    @property
-    def fail_before_sqla_084(self):
-        return exclusions.fails_if(
-            lambda config: not util.sqla_084,
-            "SQLAlchemy 0.8.4 or greater required"
-        )
-
-    @property
-    def fail_before_sqla_09(self):
-        return exclusions.fails_if(
-            lambda config: not util.sqla_09,
-            "SQLAlchemy 0.9.0 or greater required"
-        )
-
     @property
     def fail_before_sqla_100(self):
         return exclusions.fails_if(
             lambda config: not util.sqla_100,
-            "SQLAlchemy 1.0.0 or greater required"
+            "SQLAlchemy 1.0.0 or greater required",
+        )
+
+    @property
+    def fail_before_sqla_1010(self):
+        return exclusions.fails_if(
+            lambda config: not util.sqla_1010,
+            "SQLAlchemy 1.0.10 or greater required",
         )
 
     @property
     def fail_before_sqla_099(self):
         return exclusions.fails_if(
             lambda config: not util.sqla_099,
-            "SQLAlchemy 0.9.9 or greater required"
+            "SQLAlchemy 0.9.9 or greater required",
         )
 
     @property
     def fail_before_sqla_110(self):
         return exclusions.fails_if(
             lambda config: not util.sqla_110,
-            "SQLAlchemy 1.1.0 or greater required"
-        )
-
-    @property
-    def sqlalchemy_08(self):
-
-        return exclusions.skip_if(
-            lambda config: not util.sqla_08,
-            "SQLAlchemy 0.8.0b2 or greater required"
-        )
-
-    @property
-    def sqlalchemy_09(self):
-        return exclusions.skip_if(
-            lambda config: not util.sqla_09,
-            "SQLAlchemy 0.9.0 or greater required"
+            "SQLAlchemy 1.1.0 or greater required",
         )
 
     @property
     def sqlalchemy_092(self):
         return exclusions.skip_if(
             lambda config: not util.sqla_092,
-            "SQLAlchemy 0.9.2 or greater required"
+            "SQLAlchemy 0.9.2 or greater required",
         )
 
     @property
     def sqlalchemy_094(self):
         return exclusions.skip_if(
             lambda config: not util.sqla_094,
-            "SQLAlchemy 0.9.4 or greater required"
+            "SQLAlchemy 0.9.4 or greater required",
+        )
+
+    @property
+    def sqlalchemy_099(self):
+        return exclusions.skip_if(
+            lambda config: not util.sqla_099,
+            "SQLAlchemy 0.9.9 or greater required",
+        )
+
+    @property
+    def sqlalchemy_100(self):
+        return exclusions.skip_if(
+            lambda config: not util.sqla_100,
+            "SQLAlchemy 1.0.0 or greater required",
+        )
+
+    @property
+    def sqlalchemy_1014(self):
+        return exclusions.skip_if(
+            lambda config: not util.sqla_1014,
+            "SQLAlchemy 1.0.14 or greater required",
+        )
+
+    @property
+    def sqlalchemy_1115(self):
+        return exclusions.skip_if(
+            lambda config: not util.sqla_1115,
+            "SQLAlchemy 1.1.15 or greater required",
         )
 
     @property
     def sqlalchemy_110(self):
         return exclusions.skip_if(
             lambda config: not util.sqla_110,
-            "SQLAlchemy 1.1.0 or greater required"
+            "SQLAlchemy 1.1.0 or greater required",
         )
+
+    @property
+    def sqlalchemy_issue_4436(self):
+        def check(config):
+            vers = sqla_compat._vers
+
+            if vers == (1, 3, 0, 'b1'):
+                return True
+            elif vers >= (1, 2, 16):
+                return False
+            else:
+                return True
+
+        return exclusions.skip_if(
+            check,
+            "SQLAlchemy 1.2.16, 1.3.0b2 or greater required",
+        )
+
+    @property
+    def python3(self):
+        return exclusions.skip_if(
+            lambda: sys.version_info < (3,), "Python version 3.xx is required."
+        )
+
+    @property
+    def pep3147(self):
+
+        return exclusions.only_if(lambda config: util.compat.has_pep3147())
+
+    @property
+    def comments(self):
+        return exclusions.only_if(
+            lambda config: sqla_compat._dialect_supports_comments(
+                config.db.dialect
+            )
+        )
+
+    @property
+    def comments_api(self):
+        return exclusions.only_if(lambda config: util.sqla_120)
